@@ -16,9 +16,9 @@
 #    along with Gnomolicious; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 #
-#    (C) 2005 Nicolas Évrard <nicoe@nutellux.be>
+#    (C) 2005-2006 Nicolas Évrard <nicoe@nutellux.be>
 
-__revision__ = "$Id: gnomolicious.py,v 1.20 2006/02/28 22:15:27 nicoe Exp $"
+__revision__ = "$Id: gnomolicious.py,v 1.22 2006/03/21 20:05:27 nicoe Exp $"
 
 import os.path
 import re
@@ -38,6 +38,7 @@ import gconf
 import gobject
 
 import delicious
+from dbusapi import GnomoliciousDBusService
 import conf
 
 logger = logging.getLogger('gnomolicious')
@@ -118,13 +119,14 @@ class GnomoliciousApplet(gnomeapplet.Applet):
     def __init__(self, applet, iid):
         self._last = datetime.datetime.min
         self.conf = GnomoliciousConfiguration()
-        self.delicious_user = delicious.User(self.conf.username,
-                                             self.conf.password)
 
         #############################################
         #        GNOME initialization stuffs        #
         #############################################
         self.__gobject_init__()
+        self.delicious_user = delicious.User(self.conf.username,
+                                             self.conf.password)
+        self.dbusapi = GnomoliciousDBusService(self)
 
         # Popup-menu definition and popup-menu callbacks
         self.propxml = """\
@@ -228,14 +230,12 @@ class GnomoliciousApplet(gnomeapplet.Applet):
             self._last = datetime.datetime.today()
             logger.info('Updating your tags')
             self.tags.clear()
-            taglist = self.delicious_user.tags
-            for tagdict in taglist:
-                logger.debug('Adding: %s' % tagdict['tag'])
-                if int(tagdict['count']) > 1:
-                    numtag = '%s occurences' % tagdict['count']
+            for tag in self.delicious_user.tags:
+                if tag.count > 1:
+                    numtag = _('%s occurences') % tag.count
                 else:
-                    numtag = '%s occurence' % tagdict['count']
-                self.tags.append([tagdict['tag'], numtag])
+                    numtag = _('%s occurence') % tag.count
+                self.tags.append((tag.name, numtag))
             logger.info('Updating your tags: done')
 
     def clear_postwin(self):
